@@ -1,7 +1,7 @@
 package BenchmarkAnything::Storage::Backend::SQL::Query::common;
 our $AUTHORITY = 'cpan:TAPPER';
 # ABSTRACT: BenchmarkAnything::Storage::Backend::SQL - querying - backend base class
-$BenchmarkAnything::Storage::Backend::SQL::Query::common::VERSION = '0.023';
+$BenchmarkAnything::Storage::Backend::SQL::Query::common::VERSION = '0.024';
 use strict;
 use warnings;
 use base 'BenchmarkAnything::Storage::Backend::SQL::Query';
@@ -26,7 +26,7 @@ sub default_columns {
 }
 
 sub benchmark_operators {
-    return ( '=', '!=', 'like', 'not like', '<', '>', '<=', '>=' );
+    return ( '=', '!=', 'like', 'not_like', 'is_empty', '<', '>', '<=', '>=' );
 }
 
 sub create_where_clause {
@@ -34,12 +34,47 @@ sub create_where_clause {
     my ( $or_self, $s_column_name, $ar_value ) = @_;
 
     my $s_where_clause = q##;
-    if ( $ar_value->[0] eq 'not like' ) {
+    if ( $ar_value->[0] eq 'not like' or $ar_value->[0] eq 'not_like' ) {
         $s_where_clause = "$s_column_name NOT LIKE ?";
     }
     elsif ( $ar_value->[0] eq 'like' ) {
         $s_where_clause = "$s_column_name LIKE ?";
     }
+    elsif ( $ar_value->[0] eq 'is_empty' ) {
+        my $empty_option = $ar_value->[1];
+        use Data::Dumper;
+        warn "ar_value: ".Dumper($ar_value);
+        if (defined($ar_value->[1]) and $ar_value->[1] eq '0') {
+            # check that field is NOT EMPTY:
+            #   [ "is_empty", "some_field_name", 0 ]
+            $s_where_clause  = "$s_column_name IS NOT NULL AND $s_column_name != ''";
+
+        } elsif (defined($ar_value->[1]) and $ar_value->[1] eq '2') {
+            # check that field is just EMPTY but exists (ie. not undefined/null):
+            #   [ "is_empty", "some_field_name", 2 ]
+            $s_where_clause = "$s_column_name IS NULL";
+
+        } elsif (defined($ar_value->[1]) and $ar_value->[1] eq '1') { # TODO: Does not work yet (sic, THE actual feature)
+            # check that field is EMPTY or UNDEFINED/NULL:
+            #   [ "is_empty", "some_field_name", 1 ]
+            #   [ "is_empty", "some_field_name" ]
+            $s_where_clause = "$s_column_name IS NULL OR $s_column_name = ''";
+
+        } elsif (not defined($ar_value->[1])) {
+            # check that field is EMPTY or UNDEFINED/NULL:
+            #   [ "is_empty", "some_field_name", 1 ]
+            #   [ "is_empty", "some_field_name" ]
+            warn "unsupported 'is_empty' condition (undef). Interpreting as 'is_empty' condition (1).";
+
+        } else {
+            # we might invent other semantics so we better warn about
+            # what could once become meaningful.
+            warn "unclear 'is_empty' condition (".$ar_value->[1]."). Interpreting as 'is_empty' condition (1).";
+            $s_where_clause = "$s_column_name IS NULL OR $s_column_name = ''";
+        }
+        warn "WHERE_CLAUSE: ".$s_where_clause."\n";
+        warn "ar_value->[1]:  ".$ar_value->[1]."\n";
+    }
     elsif (
            $ar_value->[0] eq '<'
         || $ar_value->[0] eq '>'
@@ -617,11 +652,26 @@ sub update_raw_bench_bundle_set_processed {
         UPDATE raw_bench_bundles
         SET processed=1,
             processing=0
-        WHERE raw_bench_bundle_id = ?
+        WHERE processed=0 AND
+              processing=1 AND
+              raw_bench_bundle_id = ?
     ", @a_vals );
 
 }
 
+# sub unlock_raw_bench_bundle {
+
+#     my ( $or_self, @a_vals ) = @_;
+
+#     return $or_self->execute_query( "
+#         UPDATE raw_bench_bundles
+#         SET processing = 0
+#         WHERE processed = 0  AND
+#               processing = 1 AND
+#               raw_bench_bundle_id = ?
+#     ", @a_vals );
+# }
+
 sub delete_benchmark_additional_relations {
 
     my ( $or_self, @a_vals ) = @_;
@@ -728,7 +778,7 @@ Roberto Schaefer <schaefr@amazon.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2017 by Amazon.com, Inc. or its affiliates.
+This software is Copyright (c) 2018 by Amazon.com, Inc. or its affiliates.
 
 This is free software, licensed under:
 
